from django.core.management.base import BaseCommand
from souschef.order.models import Order
from souschef.member.models import Client
from datetime import datetime, timedelta
from django.contrib.admin.models import LogEntry, ADDITION


class Command(BaseCommand):
    help = 'Create new orders for a given delivery date for all\
            ongoing active clients using their meals defaults.'

    def add_arguments(self, parser):
        parser.add_argument(
            'delivery_date',
            help='The date must be in the format YYYY-MM-DD',
        )
        parser.add_argument(
            '--days',
            help=(
                'The number of days to be created in advance.'
                'There will not be duplicates.'
            ),
            default=1,
            type=int
        )

    def handle(self, *args, **options):
        start_date = datetime.strptime(
            options['delivery_date'], '%Y-%m-%d'
        ).date()
        days = options['days']

        # Only active ongoing clients can receive orders
        clients = Client.ongoing.all()

        for i in range(days):
            # Create the orders
            delivery_date = start_date + timedelta(days=i)
            orders = Order.objects.auto_create_orders(
                delivery_date, clients
            )
            # Log the execution
            LogEntry.objects.log_action(
                user_id=1, content_type_id=1,
                object_id="", object_repr="Orders creation for "+str(
                    delivery_date.strftime('%Y-%m-%d %H:%M')),
                action_flag=ADDITION,
            )
            print("{0} orders created on {1}: to be delivered on {2}.".format(
                len(orders), start_date, delivery_date
            ))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-04-02 19:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0016_auto_20180704_1613'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order_item',
            name='total_quantity',
            field=models.IntegerField(default=0, verbose_name='total quantity'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-23 01:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0003_auto_20160615_1504'),
    ]

    operations = [
        migrations.AddField(
            model_name='order_item',
            name='free_quantity',
            field=models.IntegerField(null=True, verbose_name='free quantity'),
        ),
        migrations.AddField(
            model_name='order_item',
            name='total_quantity',
            field=models.IntegerField(null=True, verbose_name='total quantity'),
        ),
        migrations.AlterField(
            model_name='order',
            name='status',
            field=models.CharField(choices=[('O', 'Ordered'), ('D', 'Delivered'), ('B', 'Billed'), ('P', 'Paid')], max_length=1, verbose_name='order status'),
        ),
    ]



from django.db import models
from django.db.models import Q
from django_filters import ChoiceFilter, DateFromToRangeFilter, FilterSet, CharFilter
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.utils import timezone


# Create your models here.

class NoteManager(models.Manager):
    pass


class UnreadNoteManager(NoteManager):

    def get_queryset(self):

        return super(UnreadNoteManager, self).get_queryset().filter(
            is_read=0
        )


class NotePriority(models.Model):
    name = models.CharField(max_length=150, verbose_name=_('Name'))

    class Meta:
        verbose_name_plural = _('Note priorities')
        ordering = ('name',)

    def __str__(self):
        return u"%s" % self.name


class NoteCategory(models.Model):
    name = models.CharField(max_length=150, verbose_name=_('Name'))

    class Meta:
        verbose_name_plural = _('Note categories')
        ordering = ('name',)

    def __str__(self):
        return u"%s" % self.name


class Note(models.Model):

    class Meta:
        verbose_name_plural = _('Notes')
        ordering = ('-date_modified', 'note')

    note = models.TextField(
        verbose_name=_('Note')
    )

    author = models.ForeignKey(
        User,
        verbose_name=_('Author'),
        null=True,
        on_delete=models.SET_NULL
    )

    date_created = models.DateTimeField(
        verbose_name=_('Date Created'),
        default=timezone.now,
    )

    date_modified = models.DateTimeField(
        verbose_name=_('Date Modified'),
        default=timezone.now,
    )

    is_read = models.BooleanField(
        verbose_name=_('Is read'),
        default=False
    )

    client = models.ForeignKey(
        'member.Client',
        verbose_name=_('Client'),
        related_name='client_notes',
        on_delete=models.CASCADE
    )

    priority = models.ForeignKey(
        NotePriority,
        verbose_name=_('Priority'),
        related_name="notes",
        null=True,
        on_delete=models.SET_NULL
    )

    category = models.ForeignKey(
        NoteCategory,
        verbose_name=_('Category'),
        related_name="notes",
        null=True,
        on_delete=models.SET_NULL
    )

    is_deleted = models.BooleanField(
        verbose_name=_('Is deleted'),
        default=False
    )

    objects = NoteManager()

    unread = UnreadNoteManager()

    def __str__(self):
        return self.note

    def mark_as_read(self):
        """ Mark a note as read. """
        if not self.is_read:
            self.is_read = True
            self.save()

    def mark_as_unread(self):
        """ Mark a note as unread. """
        if self.is_read:
            self.is_read = False
            self.save()

    def delete(self):
        """ Mark a note as being deleted. """
        self.is_deleted = True
        self.save()


class NoteFilter(FilterSet):

    IS_READ_CHOICES = (
        ('', 'All'),
        ('1', 'Yes'),
        ('0', 'No'),
    )

    NOTE_STATUS_UNREAD = IS_READ_CHOICES[2][0]

    is_read = ChoiceFilter(
        choices=IS_READ_CHOICES,
    )

    name = CharFilter(
        method='filter_search',
        label=_('Search by name')
    )

    date_modified = DateFromToRangeFilter(lookup_expr='contains')

    class Meta:
        model = Note
        fields = ['priority', 'is_read', 'date_modified', 'category', ]

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        names = value.split(' ')

        name_contains = Q()

        for name in names:
            firstname_contains = Q(
                client__member__firstname__icontains=name
            )

            name_contains |= firstname_contains

            lastname_contains = Q(
                client__member__lastname__icontains=name
            )
            name_contains |= lastname_contains

        return queryset.filter(name_contains)



COMPONENT_SYSTEM_DEFAULT = {
    'main_dish': 1,
    'size': 'R',
    'dessert': 1,
    'diabetic': 0,
    'fruit_salad': 0,
    'green_salad': 0,
    'pudding': 0,
    'compote': 0,
    'sides': 0
}



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone


class Notification(models.Model):

    class Meta:
        verbose_name_plural = _('notifications')

    description = models.TextField(
        verbose_name=_('description')
    )

    member = models.ForeignKey(
        'member.Member',
        verbose_name=_('member'),
        on_delete=models.CASCADE
    )

    date = models.DateField(
        auto_now=False,
        auto_now_add=False,
        default=timezone.now
    )

    def __str__(self):
        return self.description



from datetime import date, datetime

from django.core.management.base import BaseCommand
from souschef.member.models import ClientScheduledStatus


class Command(BaseCommand):
    help = 'Process scheduled status changes, \
        queued in «member_ClientScheduledStatus» table.'

    def handle(self, *args, **options):
        # List all change not processed, and older or equal to now
        changes = ClientScheduledStatus.objects.filter(
            operation_status=ClientScheduledStatus.TOBEPROCESSED
        ).filter(
            change_date__lte=date.today()
        )

        # For each change to be processed,
        for scheduled_change in changes:
            if scheduled_change.process():
                suc_msg = ": client «{}» status updated from {} to {}.".format(
                    scheduled_change.client.member,
                    scheduled_change.get_status_from_display(),
                    scheduled_change.get_status_to_display()
                )
                self.stdout.write(self.style.SUCCESS(
                    str(datetime.now()) + suc_msg))
            # If not, mark change as processed with error
            else:
                err_msg = ': client «{}» status not updated.'.format(
                    scheduled_change.client.member
                )
                err_msg += ' Current status is «{}», should be «{}».'.format(
                    scheduled_change.client.get_status_display(),
                    scheduled_change.get_status_from_display()
                )
                self.stdout.write(self.style.ERROR(
                    str(datetime.now()) + err_msg))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-05 21:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0001_fix161f'),
    ]

    operations = [
        migrations.AddField(
            model_name='ingredient',
            name='ingredient_group',
            field=models.CharField(choices=[('meat', 'Meat'), ('dairy', 'Dairy'), ('fish', 'Fish'), ('seafood', 'Seafood'), ('veggies and fruits', 'Veggies and fruits'), ('legumineuse', 'Legumineuse'), ('grains', 'Grains'), ('fresh herbs', 'Fresh herbs'), ('spices', 'Spices'), ('dry and canned goods', 'Dry and canned goods'), ('oils and sauces', 'Oils and sauces')], default='', max_length=100, verbose_name='ingredient group'),
            preserve_default=False,
        ),
    ]



import collections
from django.db import models
from django.db.models import Q, Prefetch
from souschef.order.models import Order, Order_item
from datetime import datetime, date
from annoying.fields import JSONField
from django.utils.translation import ugettext_lazy as _
from django_filters import FilterSet, CharFilter


class BillingManager(models.Manager):

    def billing_create_new(self, year, month):
        """
        Create a new billing for the given period.
        A period is a month.
         """
        # Get all billable orders for the given period
        billable_orders = Order.objects.get_billable_orders(
            year, month
        ).select_related(
            'client__member'
        ).only(
            'client__member__firstname',
            'client__member__lastname'
        ).prefetch_related(Prefetch(
            'orders',
            queryset=Order_item.objects.all().only(
                'order__id',
                'price',
                'billable_flag'
            )
        ))

        total_amount = calculate_amount_total(billable_orders)

        # Create the Billing object
        billing = Billing.objects.create(
            total_amount=total_amount,
            billing_month=month,
            billing_year=year,
            created=datetime.today(),
            detail={},
        )

        # Attach the orders
        billing.orders.add(*billable_orders)

        return billing

    def billing_get_period(self, year, month):
        """
        Check if a billing exists for a given period.
        Return None otherwise.
        """
        billing = Billing.objects.filter(
            billing_year=year,
            billing_month=month,
        )

        if billing.count() == 0:
            return None
        else:
            return billing


class Billing(models.Model):

    class Meta:
        ordering = ["-billing_year", "-billing_month"]

    total_amount = models.DecimalField(
        verbose_name=_('total_amount'),
        max_digits=8,
        decimal_places=2
    )

    # Month start with january is 1
    billing_month = models.IntegerField()

    billing_year = models.IntegerField()

    created = models.DateTimeField(
        verbose_name=None, auto_now=True
    )

    detail = JSONField()

    orders = models.ManyToManyField(Order)

    objects = BillingManager()

    @property
    def billing_period(self):
        """
        Return a readable format for the billing period.
        """
        period = date(self.billing_year, self.billing_month, 1)
        return period

    @property
    def summary(self):
        """
        Return a summary of every client.
        Format: dictionary {client: info}
        """
        # collect orders by clients
        kvpairs = map(
            lambda o: (o.client, o),
            self.orders.all()
        )
        d = collections.defaultdict(list)
        for k, v in kvpairs:
            d[k].append(v)
        result = {}
        for client, orders in d.items():
            result[client] = {
                'total_orders': len(orders),
                'total_main_dishes': {
                    'R': 0,  # to be counted
                    'L': 0
                },
                'total_billable_sides': 0,  # to be counted
                'total_amount': sum(map(lambda o: o.price, orders))
            }
            for o in orders:
                for o_item in o.orders.all():
                    if o_item.component_group == 'main_dish':
                        if o_item.size == 'R':
                            result[client][
                                'total_main_dishes'
                            ]['R'] += o_item.total_quantity
                        elif o_item.size == 'L':
                            result[client][
                                'total_main_dishes'
                            ]['L'] += o_item.total_quantity
                    else:
                        if o_item.billable_flag is True:
                            result[client][
                                'total_billable_sides'
                            ] += o_item.total_quantity
        return result


class BillingFilter(FilterSet):
    name = CharFilter(
        method='filter_search',
        label=_('Search by name')
    )

    date = CharFilter(
        method='filter_period'
    )

    class Meta:
        model = Billing
        fields = '__all__'

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        name_contains = Q()
        names = value.split(' ')

        for name in names:

            firstname_contains = Q(
                client__member__firstname__icontains=name
            )

            lastname_contains = Q(
                client__member__lastname__icontains=name
            )

            name_contains |= firstname_contains | lastname_contains

        return queryset.filter(name_contains)

    def filter_period(self, queryset, field_name, value):
        if not value:
            return queryset

        year, month = value.split('-')
        return queryset.filter(billing_year=year, billing_month=month)


# get the total amount from a list of orders
def calculate_amount_total(orders):
    total = 0
    for order in orders:
        total += order.price
    return total



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.delivery.views import (Orderlist, MealInformation, RoutesInformation,
                            KitchenCount, MealLabels, DeliveryRouteSheet,
                            RefreshOrderView, CreateDeliveryOfToday,
                            EditDeliveryOfToday)

app_name = "delivery"

urlpatterns = [
    url(_(r'^order/$'), Orderlist.as_view(), name='order'),
    url(_(r'^meal/$'), MealInformation.as_view(), name='meal'),
    url(_(r'^meal/(?P<id>\d+)/$'), MealInformation.as_view(), name='meal_id'),
    url(_(r'^routes/$'), RoutesInformation.as_view(), name='routes'),
    url(_(r'^route/(?P<pk>\d+)/$'),
        EditDeliveryOfToday.as_view(), name='edit_delivery_of_today'),
    url(_(r'^route/(?P<pk>\d+)/create/$'),
        CreateDeliveryOfToday.as_view(), name='create_delivery_of_today'),
    url(_(r'^kitchen_count/$'), KitchenCount.as_view(), name='kitchen_count'),
    url(_(r'^kitchen_count/(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d+)/$'),
        KitchenCount.as_view(), name='kitchen_count_date'),
    url(_(r'^viewDownloadKitchenCount/$'),
        KitchenCount.as_view(), name='downloadKitchenCount'),
    url(_(r'^viewMealLabels/$'), MealLabels.as_view(), name='mealLabels'),
    url(_(r'^route_sheet/(?P<pk>\d+)/$'),
        DeliveryRouteSheet.as_view(), name='route_sheet'),
    url(_(r'^refresh_orders/$'),
        RefreshOrderView.as_view(), name='refresh_orders'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-30 15:34
from __future__ import unicode_literals

import annoying.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0005_fix004a'),
    ]

    operations = [
        migrations.AddField(
            model_name='client',
            name='meal_default_week',
            field=annoying.fields.JSONField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Django settings for sous-chef project.

Generated by 'django-admin startproject' using Django 1.9.2.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

import os
import subprocess
from django.urls import reverse_lazy

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get(
    'SOUSCHEF_DJANGO_SECRET_KEY',
    '15ine$#^qas4_h2u7yk&lxat*&g*b8+)@wp$2x@vi2#v9)i2#u')  # For dev purposes

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.environ.get('SOUSCHEF_ENVIRONMENT_NAME') == 'DEV'

# This IP may change for different computers and should be the
# request.META.get('REMOTE_ADDR') for your local computer.
# Don't run with this in production and don't
# commit any changes to this INTERNAL_IPS settings.
# WE NEED THIS IN ORDER TO USE 'debug' IN TEMPLATES!!!
INTERNAL_IPS = os.environ.get(
    'SOUSCHEF_DJANGO_INTERNAL_IPS',
    # Default value is an example: use space-separated string in env
    '172.19.0.1  172.22.0.1').strip().split()

ALLOWED_HOSTS = os.environ.get(
    'SOUSCHEF_DJANGO_ALLOWED_HOSTS',
    # Default value is an example: use space-separated string in env
    'localhost souschef.example.com test.souschef.example.com').strip().split()


# Application definition

INSTALLED_APPS = [
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # Third party apps
    'annoying',
    'avatar',
    'django_crontab',
    'django_filters',
    'formtools',
    'leaflet',
    'rules.apps.AutodiscoverRulesConfig',

    # Sous-chef apps
    'souschef.sous_chef',
    'souschef.billing',
    'souschef.datamigration',
    'souschef.delivery',
    'souschef.meal',
    'souschef.member',
    'souschef.order',
    'souschef.notification',
    'souschef.page',
    'souschef.note',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
]

ROOT_URLCONF = 'souschef.sous_chef.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            BASE_DIR + '/sous_chef/templates/'
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.i18n',
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'souschef.sous_chef.context_processors.total',
            ],
        },
    },
]

WSGI_APPLICATION = 'souschef.sous_chef.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': os.environ.get('SOUSCHEF_DJANGO_DB_NAME', 'feast'),
        'USER': os.environ.get('SOUSCHEF_DJANGO_DB_USER', 'root'),
        'PASSWORD': os.environ.get('SOUSCHEF_DJANGO_DB_PASSWORD', '123456'),
        'HOST': os.environ.get('SOUSCHEF_DJANGO_DB_HOST', 'db'),
        'port': os.environ.get('SOUSCHEF_DJANGO_DB_PORT', '3306'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.'
                'NumericPasswordValidator',
    },
]

LOGIN_URL = reverse_lazy('page:login')
LOGIN_REDIRECT_URL = reverse_lazy('page:home')

AUTHENTICATION_BACKENDS = (
    'rules.permissions.ObjectPermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
)


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en'
USE_I18N = True
USE_L10N = True

# Store datetimes as UTC in database
USE_TZ = True

# Use this timezone when displaying datetimes
TIME_ZONE = 'America/Montreal'

# List of supported languages
LANGUAGES = (
    ('fr', 'Français'),
    ('en', 'English'),
)

LOCALE_PATHS = tuple(map(
    lambda path: os.path.join(BASE_DIR, path),
    (
        'meal/locale/',
        'member/locale/',
        'notification/locale/',
        'order/locale/',
        'page/locale/',
        'delivery/locale/',
        'billing/locale',
        'note/locale',
    ),
))

FORMAT_MODULE_PATH = (
    'souschef.sous_chef.formats',
)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

STATICFILES_DIRS = (
    BASE_DIR + '/sous_chef/assets/',
)
STATIC_URL = '/static/'

GENERATED_DOCS_DIR = os.environ.get(
    'SOUSCHEF_GENERATED_DOCS_DIR',
    '/var/local/souschef')

CRONJOBS = [
    ('0 0 * * *', 'souschef.pycrons.cleaning.clean_old_pdf_files')
]

if DEBUG:
    # When using the development server, serve files directly from /media/
    # https://docs.djangoproject.com/en/1.11/howto/static-files/#serving-files-uploaded-by-a-user-during-development
    MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
    MEDIA_URL = '/media/'
    GENERATED_DOCS_DIR = os.environ.get(
        'SOUSCHEF_GENERATED_DOCS_DIR', BASE_DIR)
else:
    # In non development mode, serve files from /static/ using nginx as
    # dedicated server
    # https://docs.djangoproject.com/en/1.11/howto/static-files/deployment/#serving-static-files-from-a-dedicated-server
    MEDIA_ROOT = os.path.join(STATIC_ROOT, 'media')
    MEDIA_URL = '/static/media/'

# Avatar files
AVATAR_PROVIDERS = (
    'avatar.providers.PrimaryAvatarProvider',
    'avatar.providers.GravatarAvatarProvider',
    'avatar.providers.DefaultAvatarProvider',
)

MEAL_LABELS_FILE = os.path.join(GENERATED_DOCS_DIR, "meal_labels.pdf")
KITCHEN_COUNT_FILE = os.path.join(GENERATED_DOCS_DIR, "kitchen_count.pdf")
ROUTE_SHEETS_FILE = os.path.join(GENERATED_DOCS_DIR, "route_sheets.pdf")

# Displayable information
SOUSCHEF_ENVIRONMENT_NAME = os.environ.get('SOUSCHEF_ENVIRONMENT_NAME') or ''

try:
    GIT_HEAD = subprocess.check_output(['git', 'rev-parse', 'HEAD'])
    GIT_TAG = subprocess.check_output(['git', 'describe', '--tags'])
except Exception:
    GIT_HEAD = None
    GIT_TAG = None



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-11-22 05:15
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0002_auto_20161122_0458'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='billing',
            options={'ordering': ['billing_year', '-billing_month']},
        ),
    ]



from django import forms
from souschef.member.models import Client
from souschef.note.models import Note


class NoteForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(NoteForm, self).__init__(*args, **kwargs)
        self.fields['client'].queryset = Client.objects.all().select_related(
            'member'
        ).only(
            'member__firstname',
            'member__lastname'
        )

    class Meta:
        model = Note
        fields = ['note', 'client', 'priority', 'category']

        widgets = {
            'note': forms.Textarea(attrs={'rows': 7}),
            'client': forms.Select(
                attrs={'class': 'ui search dropdown'}
            ),
            'priority': forms.Select(
                attrs={'class': 'ui dropdown'}
            ),
            'category': forms.Select(
                attrs={'class': 'ui dropdown'}
            ),
        }

class NoteEditForm(forms.ModelForm):
    class Meta:
        model = Note
        fields = ['note', 'priority', 'category']

        widgets = {
            'note': forms.Textarea(attrs={'rows': 7}),
            'priority': forms.Select(
                attrs={'class': 'ui dropdown'}
            ),
            'category': forms.Select(
                attrs={'class': 'ui dropdown'}
            ),
        }



from .settings import *  # noqa


SECRET_KEY = "SecretKeyForUseOnTravis"

DEBUG = False
TEMPLATE_DEBUG = False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-05 19:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('member', '0001_fix161f'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.TextField(verbose_name='description')),
                ('date', models.DateField(default=django.utils.timezone.now)),
                ('member', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Member', verbose_name='member')),
            ],
            options={
                'verbose_name_plural': 'notifications',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-01-15 04:19
from __future__ import unicode_literals

from django.db import migrations, models


def set_initial_priorities(apps, schema_editor):
    # set NotePriority initial
    NotePriority = apps.get_model('note', 'NotePriority')
    NotePriority.objects.create(name="Normal")
    NotePriority.objects.create(name="Urgent")


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0003_auto_20160819_2037'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotePriority',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='Name')),
            ],
            options={
                'ordering': ('name',), 'verbose_name_plural': 'Note priorities'
            },
        ),
        migrations.RunPython(set_initial_priorities),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-05 19:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('member', '0001_fix161f'),
        ('meal', '0001_fix161f'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_date', models.DateField(verbose_name='creation date')),
                ('delivery_date', models.DateField(verbose_name='delivery date')),
                ('status', models.CharField(choices=[('', 'Order status'), ('O', 'Ordered'), ('D', 'Delivered'), ('B', 'Billed'), ('P', 'Paid')], max_length=1, verbose_name='order status')),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Client', verbose_name='client')),
            ],
            options={
                'verbose_name_plural': 'orders',
            },
        ),
        migrations.CreateModel(
            name='Order_item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.DecimalField(decimal_places=2, max_digits=6, verbose_name='price')),
                ('billable_flag', models.BooleanField(verbose_name='billable flag')),
                ('size', models.CharField(choices=[('', 'Serving size'), ('R', 'Regular'), ('L', 'Large')], max_length=1, null=True, verbose_name='size')),
                ('order_item_type', models.CharField(choices=[('', 'Order item type'), ('B component', 'BILLABLE meal component (main dish, vegetable, side dish, seasonal)'), ('B delivery', 'BILLABLE delivery (general store item, ...)'), ('N delivery', 'NON BILLABLE delivery (ex. invitation card, ...)'), ('N pickup', 'NON BILLABLE pickup (payment)')], max_length=20, verbose_name='order item type')),
                ('remark', models.CharField(max_length=256, verbose_name='remark')),
                ('component', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='meal.Component', verbose_name='component')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='order.Order', verbose_name='order')),
            ],
            options={
                'verbose_name_plural': 'order items',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-03-05 16:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0037_one_address_per_member'),
    ]

    operations = [
        migrations.AlterField(
            model_name='member',
            name='firstname',
            field=models.CharField(max_length=50, verbose_name='First name'),
        ),
        migrations.AlterField(
            model_name='member',
            name='lastname',
            field=models.CharField(max_length=50, verbose_name='Last name'),
        ),
    ]



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="souschef",
    version="1.3.6",
    license="AGPL-3.0",
    author="Santropol Roulant and Savoir Faire Linux",
    author_email="info@santropolroulant.org",
    description="Webapp used to manage orders for meals-on-wheel delivery",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sous-chef",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta ",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Content Management System",
    ],
    keywords="meals-on-wheel",
    python_requires='>=3.7',
    install_requires=[
        'django-annoying>=0.10,<0.10.999',
        'django-avatar>=4.0,<4.0.999',
        'django-crontab>=0.7.1,<0.7.999',
        'django-extra-views>=0.9,<0.9.999',
        'django-filter>=1.0,<1.0.999',
        'django-formtools>=2.0,<2.0.999',
        'django-leaflet>=0.22,<0.22.999',
        'django-localflavor>=1.5,<1.5.999',
        'django-template-i18n-lint>=1.2,<1.2.999',
        'django>=1.11,<1.11.999',
        'factory_boy>=2.8,<2.8.999',
        'mysqlclient>=1.3,<1.3.999',
        'pylabels>=1.2,<1.2.999',
        'reportlab>=3.4,<3.4.999',
        'rules>=1.2,<1.2.999',
        'transifex-client>=0.12,<0.12.999',
    ],
)



import csv
from datetime import date

from django.core.management.base import BaseCommand

from souschef.member.models import (
    Client, Member, Referencing, EmergencyContact
)


class Command(BaseCommand):
    help = 'Data: import clients relationships from given csv file.'

    ROW_MID = 0
    ROW_RID = 1
    ROW_FIRSTNAME = 2
    ROW_LASTNAME = 3
    ROW_EMERGENCY = 4
    ROW_REFERENT = 5
    ROW_BILLTO = 6
    ROW_WORK_INFORMATION = 7
    ROW_RELATIONSHIP = 8
    ROW_REASON = 9
    ROW_STREET = 10
    ROW_APARTMENT = 11
    ROW_POSTAL_CODE = 12
    ROW_CITY = 13
    ROW_PHONE = 14

    def add_arguments(self, parser):
        parser.add_argument(
            '--file',
            default=False,
            help='Import mock data instead of actual data',
        )

    def handle(self, *args, **options):
        if options['file']:
            file = 'mock_relationships.csv'
        else:
            file = 'clients_relationships.csv'

        with open(file) as f:
            reader = csv.reader(f, delimiter=';')
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    client = Client.objects.get(member=member)

                    if row[self.ROW_FIRSTNAME] != '':

                        relationship, created = \
                            Member.objects.update_or_create(
                                rid=row[self.ROW_RID], defaults={
                                    "firstname": row[
                                        self.ROW_FIRSTNAME], "lastname": row[
                                        self.ROW_LASTNAME],
                                    "work_information": row[
                                        self.ROW_WORK_INFORMATION]})

                        if row[self.ROW_EMERGENCY] == '1':
                            EmergencyContact.objects.create(
                                client=client,
                                member=relationship,
                                relationship=row[self.ROW_RELATIONSHIP]
                            )
                            self.stdout.write(
                                self.style.SUCCESS(
                                    'Added an emergency Relationship.'
                                ))
                        if row[self.ROW_BILLTO] == '1':
                            self.stdout.write(
                                self.style.SUCCESS(
                                    'Added a blling Relationship.'
                                ))
                            client.billing_member = relationship
                        if row[self.ROW_REFERENT] == '1':
                            Referencing.objects.create(
                                referent=relationship,
                                client=client,
                                referral_reason=row[self.ROW_REASON],
                                date=date.today(), )

                        client.save()

                except Member.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing member'))
                except Client.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing client'))



# -*- coding: utf-8 -*-
# flake8: noqa

from souschef.meal.models import Component, Restricted_item, Ingredient
from souschef.meal.models import Component_ingredient, Incompatibility


def print_rows(q, heading=""):
    pass
    print("\n-----------------------------------------------------\n",
          # q, "\n",
          heading)
    for row in q.all():
        print(row)


def insert_all():
    print("\nSTART dataload")

    # Rename existing ingredients, components, restricted_items

    items = Ingredient.objects.all()
    for item in items:
        item.name = "OLD"+item.name
        item.save()
        print("Ingredient item name", item.name)

    items = Component.objects.all()
    for item in items:
        item.name = "OLD"+item.name
        item.save()
        print("Component item name", item.name)

    items = Restricted_item.objects.all()
    for item in items:
        item.name = "OLD"+item.name
        item.save()
        print("Restricted_item item name", item.name)

    # -----------------------------------------------------------------------------
    ing_1 = Ingredient.objects.create(
        name='Butter',
        ingredient_group='Dairy')
    ing_2 = Ingredient.objects.create(
        name='Cheese : Blue cheese',
        ingredient_group='Dairy')
    ing_3 = Ingredient.objects.create(
        name='Cheese : Bocconcini',
        ingredient_group='Dairy')
    ing_4 = Ingredient.objects.create(
        name='Cheese : Cheddar',
        ingredient_group='Dairy')
    ing_5 = Ingredient.objects.create(
        name='Cheese : Cottage cheese',
        ingredient_group='Dairy')
    ing_6 = Ingredient.objects.create(
        name='Cheese : Feta',
        ingredient_group='Dairy')
    ing_7 = Ingredient.objects.create(
        name='Cheese : Goat cheese',
        ingredient_group='Dairy')
    ing_8 = Ingredient.objects.create(
        name='Cheese : Mozzarella',
        ingredient_group='Dairy')
    ing_9 = Ingredient.objects.create(
        name='Cheese : Parmesan',
        ingredient_group='Dairy')
    ing_10 = Ingredient.objects.create(
        name='Cheese : Ricotta',
        ingredient_group='Dairy')
    ing_11 = Ingredient.objects.create(
        name='Eggs',
        ingredient_group='Dairy')
    ing_12 = Ingredient.objects.create(
        name='Margarine',
        ingredient_group='Dairy')
    ing_13 = Ingredient.objects.create(
        name='Milk',
        ingredient_group='Dairy')
    ing_14 = Ingredient.objects.create(
        name='Tofu',
        ingredient_group='Dairy')
    ing_15 = Ingredient.objects.create(
        name='Yogurt',
        ingredient_group='Dairy')
    ing_16 = Ingredient.objects.create(
        name='Almond Powder ',
        ingredient_group='Dry and Canned goods')
    ing_17 = Ingredient.objects.create(
        name='Almonds',
        ingredient_group='Dry and Canned goods')
    ing_18 = Ingredient.objects.create(
        name='Capers',
        ingredient_group='Dry and Canned goods')
    ing_19 = Ingredient.objects.create(
        name='Corn meal',
        ingredient_group='Dry and Canned goods')
    ing_20 = Ingredient.objects.create(
        name='corn starch',
        ingredient_group='Dry and Canned goods')
    ing_21 = Ingredient.objects.create(
        name='Creamed corn',
        ingredient_group='Dry and Canned goods')
    ing_22 = Ingredient.objects.create(
        name='Egg noodles',
        ingredient_group='Dry and Canned goods')
    ing_23 = Ingredient.objects.create(
        name='Linguini',
        ingredient_group='Dry and Canned goods')
    ing_24 = Ingredient.objects.create(
        name='peanut butter',
        ingredient_group='Dry and Canned goods')
    ing_25 = Ingredient.objects.create(
        name='peanuts ',
        ingredient_group='Dry and Canned goods')
    ing_26 = Ingredient.objects.create(
        name='Pickles',
        ingredient_group='Dry and Canned goods')
    ing_27 = Ingredient.objects.create(
        name='Polenta',
        ingredient_group='Dry and Canned goods')
    ing_28 = Ingredient.objects.create(
        name='Rice noodles',
        ingredient_group='Dry and Canned goods')
    ing_29 = Ingredient.objects.create(
        name='sesame seeds',
        ingredient_group='Dry and Canned goods')
    ing_30 = Ingredient.objects.create(
        name='Sunflower seeds',
        ingredient_group='Dry and Canned goods')
    ing_31 = Ingredient.objects.create(
        name='Tomato paste',
        ingredient_group='Dry and Canned goods')
    ing_32 = Ingredient.objects.create(
        name='Walnuts',
        ingredient_group='Dry and Canned goods')
    ing_33 = Ingredient.objects.create(
        name='Whole wheat lasagna',
        ingredient_group='Dry and Canned goods')
    ing_34 = Ingredient.objects.create(
        name='Whole wheat linguini ',
        ingredient_group='Dry and Canned goods')
    ing_35 = Ingredient.objects.create(
        name='Whole wheat macaroni',
        ingredient_group='Dry and Canned goods')
    ing_36 = Ingredient.objects.create(
        name='Whole wheat penne ',
        ingredient_group='Dry and Canned goods')
    ing_37 = Ingredient.objects.create(
        name='Whole wheat spaghetti',
        ingredient_group='Dry and Canned goods')
    ing_38 = Ingredient.objects.create(
        name='Canned salmon',
        ingredient_group='Fish')
    ing_39 = Ingredient.objects.create(
        name='Canned tuna',
        ingredient_group='Fish')
    ing_40 = Ingredient.objects.create(
        name='Cod',
        ingredient_group='Fish')
    ing_41 = Ingredient.objects.create(
        name='Haddock',
        ingredient_group='Fish')
    ing_42 = Ingredient.objects.create(
        name='Pollock',
        ingredient_group='Fish')
    ing_43 = Ingredient.objects.create(
        name='Basil',
        ingredient_group='Fresh herbs')
    ing_44 = Ingredient.objects.create(
        name='Chives',
        ingredient_group='Fresh herbs')
    ing_45 = Ingredient.objects.create(
        name='Cilantro',
        ingredient_group='Fresh herbs')
    ing_46 = Ingredient.objects.create(
        name='Dill',
        ingredient_group='Fresh herbs')
    ing_47 = Ingredient.objects.create(
        name='Marjoram',
        ingredient_group='Fresh herbs')
    ing_48 = Ingredient.objects.create(
        name='Oregano',
        ingredient_group='Fresh herbs')
    ing_49 = Ingredient.objects.create(
        name='Parsley',
        ingredient_group='Fresh herbs')
    ing_50 = Ingredient.objects.create(
        name='Rosemary',
        ingredient_group='Fresh herbs')
    ing_51 = Ingredient.objects.create(
        name='Thyme',
        ingredient_group='Fresh herbs')
    ing_52 = Ingredient.objects.create(
        name='Barley (gluten)',
        ingredient_group='Grains')
    ing_53 = Ingredient.objects.create(
        name='Basmati rice',
        ingredient_group='Grains')
    ing_54 = Ingredient.objects.create(
        name='Breadcrumbs',
        ingredient_group='Grains')
    ing_55 = Ingredient.objects.create(
        name='Brown rice',
        ingredient_group='Grains')
    ing_56 = Ingredient.objects.create(
        name='Buckwheat',
        ingredient_group='Grains')
    ing_57 = Ingredient.objects.create(
        name='Couscous (gluten)',
        ingredient_group='Grains')
    ing_58 = Ingredient.objects.create(
        name='Millet',
        ingredient_group='Grains')
    ing_59 = Ingredient.objects.create(
        name='Oats',
        ingredient_group='Grains')
    ing_60 = Ingredient.objects.create(
        name='Quinoa',
        ingredient_group='Grains')
    ing_61 = Ingredient.objects.create(
        name='White flour',
        ingredient_group='Grains')
    ing_62 = Ingredient.objects.create(
        name='Whole wheat flour',
        ingredient_group='Grains')
    ing_63 = Ingredient.objects.create(
        name='Wild rice',
        ingredient_group='Grains')
    ing_64 = Ingredient.objects.create(
        name='Black beans',
        ingredient_group='Legumineuse')
    ing_65 = Ingredient.objects.create(
        name='Chickpeas',
        ingredient_group='Legumineuse')
    ing_66 = Ingredient.objects.create(
        name='Kidney beans',
        ingredient_group='Legumineuse')
    ing_67 = Ingredient.objects.create(
        name='Lentils',
        ingredient_group='Legumineuse')
    ing_68 = Ingredient.objects.create(
        name='White beans',
        ingredient_group='Legumineuse')
    ing_69 = Ingredient.objects.create(
        name='Beef cubes',
        ingredient_group='Meat')
    ing_70 = Ingredient.objects.create(
        name='Chicken',
        ingredient_group='Meat')
    ing_71 = Ingredient.objects.create(
        name='Chicken thighs',
        ingredient_group='Meat')
    ing_72 = Ingredient.objects.create(
        name='Ground beef',
        ingredient_group='Meat')
    ing_73 = Ingredient.objects.create(
        name='Ground porc',
        ingredient_group='Meat')
    ing_74 = Ingredient.objects.create(
        name='Ham',
        ingredient_group='Meat')
    ing_75 = Ingredient.objects.create(
        name='Lamb cubes',
        ingredient_group='Meat')
    ing_76 = Ingredient.objects.create(
        name='Pork sausage',
        ingredient_group='Meat')
    ing_77 = Ingredient.objects.create(
        name='Pork toulouse sausage',
        ingredient_group='Meat')
    ing_78 = Ingredient.objects.create(
        name='Roast Beef ',
        ingredient_group='Meat')
    ing_79 = Ingredient.objects.create(
        name='stock',
        ingredient_group='Meat')
    ing_80 = Ingredient.objects.create(
        name='Turkey',
        ingredient_group='Meat')
    ing_81 = Ingredient.objects.create(
        name='Turkey thighs',
        ingredient_group='Meat')
    ing_82 = Ingredient.objects.create(
        name='Apple cider vinegar',
        ingredient_group='Oils and Sauces')
    ing_83 = Ingredient.objects.create(
        name='Balsamic vinegar',
        ingredient_group='Oils and Sauces')
    ing_84 = Ingredient.objects.create(
        name='cooking wine',
        ingredient_group='Oils and Sauces')
    ing_85 = Ingredient.objects.create(
        name='Dijon mustard',
        ingredient_group='Oils and Sauces')
    ing_86 = Ingredient.objects.create(
        name='honey',
        ingredient_group='Oils and Sauces')
    ing_87 = Ingredient.objects.create(
        name='maple sirop',
        ingredient_group='Oils and Sauces')
    ing_88 = Ingredient.objects.create(
        name='Olive oil',
        ingredient_group='Oils and Sauces')
    ing_89 = Ingredient.objects.create(
        name='Prepared mustard',
        ingredient_group='Oils and Sauces')
    ing_90 = Ingredient.objects.create(
        name='Red wine (salt)',
        ingredient_group='Oils and Sauces')
    ing_91 = Ingredient.objects.create(
        name='Red wine vinegar',
        ingredient_group='Oils and Sauces')
    ing_92 = Ingredient.objects.create(
        name='Rice vinegar ',
        ingredient_group='Oils and Sauces')
    ing_93 = Ingredient.objects.create(
        name='rice wine vinegar',
        ingredient_group='Oils and Sauces')
    ing_94 = Ingredient.objects.create(
        name='Sesame oil',
        ingredient_group='Oils and Sauces')
    ing_95 = Ingredient.objects.create(
        name='Soy sauce (gluten, salt)',
        ingredient_group='Oils and Sauces')
    ing_96 = Ingredient.objects.create(
        name='Vinegar',
        ingredient_group='Oils and Sauces')
    ing_97 = Ingredient.objects.create(
        name='White wine',
        ingredient_group='Oils and Sauces')
    ing_98 = Ingredient.objects.create(
        name='Seafood medley (shrimp, mussels, calamari)',
        ingredient_group='Seafood')
    ing_99 = Ingredient.objects.create(
        name='Shrimp',
        ingredient_group='Seafood')
    ing_100 = Ingredient.objects.create(
        name='Allspice',
        ingredient_group='Spices')
    ing_101 = Ingredient.objects.create(
        name='Bay leaves',
        ingredient_group='Spices')
    ing_102 = Ingredient.objects.create(
        name='Black pepper',
        ingredient_group='Spices')
    ing_103 = Ingredient.objects.create(
        name='Cardamom',
        ingredient_group='Spices')
    ing_104 = Ingredient.objects.create(
        name='Chili',
        ingredient_group='Spices')
    ing_105 = Ingredient.objects.create(
        name='Cinnamon',
        ingredient_group='Spices')
    ing_106 = Ingredient.objects.create(
        name='Cloves',
        ingredient_group='Spices')
    ing_107 = Ingredient.objects.create(
        name='Coriander',
        ingredient_group='Spices')
    ing_108 = Ingredient.objects.create(
        name='Cumin',
        ingredient_group='Spices')
    ing_109 = Ingredient.objects.create(
        name='Curry',
        ingredient_group='Spices')
    ing_110 = Ingredient.objects.create(
        name='Ginger powder',
        ingredient_group='Spices')
    ing_111 = Ingredient.objects.create(
        name='Herbes de provence',
        ingredient_group='Spices')
    ing_112 = Ingredient.objects.create(
        name='Mustard powder',
        ingredient_group='Spices')
    ing_113 = Ingredient.objects.create(
        name='Nutmeg',
        ingredient_group='Spices')
    ing_114 = Ingredient.objects.create(
        name='Paprika',
        ingredient_group='Spices')
    ing_115 = Ingredient.objects.create(
        name='Sage',
        ingredient_group='Spices')
    ing_116 = Ingredient.objects.create(
        name='Terragon',
        ingredient_group='Spices')
    ing_118 = Ingredient.objects.create(
        name='Turmeric',
        ingredient_group='Spices')
    ing_119 = Ingredient.objects.create(
        name='Apples',
        ingredient_group='Veggies & Fruits')
    ing_120 = Ingredient.objects.create(
        name='Apricots',
        ingredient_group='Veggies & Fruits')
    ing_121 = Ingredient.objects.create(
        name='Arugula',
        ingredient_group='Veggies & Fruits')
    ing_122 = Ingredient.objects.create(
        name='Artichokes',
        ingredient_group='Veggies & Fruits')
    ing_123 = Ingredient.objects.create(
        name='Bean sprouts',
        ingredient_group='Veggies & Fruits')
    ing_124 = Ingredient.objects.create(
        name='Beets',
        ingredient_group='Veggies & Fruits')
    ing_125 = Ingredient.objects.create(
        name='Bok choy',
        ingredient_group='Veggies & Fruits')
    ing_126 = Ingredient.objects.create(
        name='Broccoli',
        ingredient_group='Veggies & Fruits')
    ing_127 = Ingredient.objects.create(
        name='Brussel sprouts',
        ingredient_group='Veggies & Fruits')
    ing_128 = Ingredient.objects.create(
        name='Cabbage',
        ingredient_group='Veggies & Fruits')
    ing_129 = Ingredient.objects.create(
        name='Carrots',
        ingredient_group='Veggies & Fruits')
    ing_130 = Ingredient.objects.create(
        name='Cauliflower',
        ingredient_group='Veggies & Fruits')
    ing_131 = Ingredient.objects.create(
        name='celery',
        ingredient_group='Veggies & Fruits')
    ing_132 = Ingredient.objects.create(
        name='Celery root',
        ingredient_group='Veggies & Fruits')
    ing_133 = Ingredient.objects.create(
        name='Chicory',
        ingredient_group='Veggies & Fruits')
    ing_134 = Ingredient.objects.create(
        name='Corn',
        ingredient_group='Veggies & Fruits')
    ing_135 = Ingredient.objects.create(
        name='Cranberries',
        ingredient_group='Veggies & Fruits')
    ing_136 = Ingredient.objects.create(
        name='Cucumber',
        ingredient_group='Veggies & Fruits')
    ing_137 = Ingredient.objects.create(
        name='Eggplant',
        ingredient_group='Veggies & Fruits')
    ing_138 = Ingredient.objects.create(
        name='Garlic',
        ingredient_group='Veggies & Fruits')
    ing_139 = Ingredient.objects.create(
        name='Ginger',
        ingredient_group='Veggies & Fruits')
    ing_140 = Ingredient.objects.create(
        name='Green beans',
        ingredient_group='Veggies & Fruits')
    ing_141 = Ingredient.objects.create(
        name='Green peas',
        ingredient_group='Veggies & Fruits')
    ing_142 = Ingredient.objects.create(
        name='Green peppers',
        ingredient_group='Veggies & Fruits')
    ing_143 = Ingredient.objects.create(
        name='Kale',
        ingredient_group='Veggies & Fruits')
    ing_144 = Ingredient.objects.create(
        name='Kolhrabi',
        ingredient_group='Veggies & Fruits')
    ing_145 = Ingredient.objects.create(
        name='Leeks',
        ingredient_group='Veggies & Fruits')
    ing_146 = Ingredient.objects.create(
        name='Lemons',
        ingredient_group='Veggies & Fruits')
    ing_147 = Ingredient.objects.create(
        name='Lettuce',
        ingredient_group='Veggies & Fruits')
    ing_148 = Ingredient.objects.create(
        name='lime',
        ingredient_group='Veggies & Fruits')
    ing_149 = Ingredient.objects.create(
        name='mung bean sprouts',
        ingredient_group='Veggies & Fruits')
    ing_150 = Ingredient.objects.create(
        name='Mushrooms',
        ingredient_group='Veggies & Fruits')
    ing_151 = Ingredient.objects.create(
        name='Olives',
        ingredient_group='Veggies & Fruits')
    ing_152 = Ingredient.objects.create(
        name='Onions',
        ingredient_group='Veggies & Fruits')
    ing_153 = Ingredient.objects.create(
        name='Orange peppers',
        ingredient_group='Veggies & Fruits')
    ing_154 = Ingredient.objects.create(
        name='parsnip',
        ingredient_group='Veggies & Fruits')
    ing_155 = Ingredient.objects.create(
        name='Potatoes',
        ingredient_group='Veggies & Fruits')
    ing_156 = Ingredient.objects.create(
        name='Pumpkin',
        ingredient_group='Veggies & Fruits')
    ing_157 = Ingredient.objects.create(
        name='Radish',
        ingredient_group='Veggies & Fruits')
    ing_158 = Ingredient.objects.create(
        name='Raisins',
        ingredient_group='Veggies & Fruits')
    ing_159 = Ingredient.objects.create(
        name='Red peppers  ',
        ingredient_group='Veggies & Fruits')
    ing_160 = Ingredient.objects.create(
        name='Snap peas',
        ingredient_group='Veggies & Fruits')
    ing_161 = Ingredient.objects.create(
        name='Snow peas',
        ingredient_group='Veggies & Fruits')
    ing_162 = Ingredient.objects.create(
        name='Spinach',
        ingredient_group='Veggies & Fruits')
    ing_163 = Ingredient.objects.create(
        name='squash',
        ingredient_group='Veggies & Fruits')
    ing_164 = Ingredient.objects.create(
        name='Sweet potatoes',
        ingredient_group='Veggies & Fruits')
    ing_165 = Ingredient.objects.create(
        name='Swiss chard',
        ingredient_group='Veggies & Fruits')
    ing_166 = Ingredient.objects.create(
        name='Tomatoes',
        ingredient_group='Veggies & Fruits')
    ing_167 = Ingredient.objects.create(
        name='Turnips',
        ingredient_group='Veggies & Fruits')
    ing_168 = Ingredient.objects.create(
        name='Yellow beans',
        ingredient_group='Veggies & Fruits')
    ing_169 = Ingredient.objects.create(
        name='zucchini',
        ingredient_group='Veggies & Fruits')
    #
    print_rows(Ingredient.objects)

    # -----------------------------------------------------------------------------
    com_1 = Component.objects.create(
        name='Beef bourguignon',
        component_group='main_dish')
    com_2 = Component.objects.create(
        name='Beef meatballs',
        component_group='main_dish')
    com_3 = Component.objects.create(
        name='Beef meatloaf',
        component_group='main_dish')
    com_4 = Component.objects.create(
        name='Beet and apple salad',
        component_group='main_dish')
    com_5 = Component.objects.create(
        name='Chicken and squash curry',
        component_group='main_dish')
    com_6 = Component.objects.create(
        name='Chicken cacciatore',
        component_group='main_dish')
    com_7 = Component.objects.create(
        name='Chicken pot pie',
        component_group='main_dish')
    com_8 = Component.objects.create(
        name='Chicken, vegetable and quinoa couscous',
        component_group='main_dish')
    com_9 = Component.objects.create(
        name='Cod cakes',
        component_group='main_dish')
    com_10 = Component.objects.create(
        name='Coq au vin',
        component_group='main_dish')
    com_11 = Component.objects.create(
        name='Creamy salmon pasta',
        component_group='main_dish')
    com_12 = Component.objects.create(
        name='Egg and salmon salad',
        component_group='main_dish')
    com_13 = Component.objects.create(
        name='Eggplant parmesan',
        component_group='main_dish')
    com_14 = Component.objects.create(
        name='Fish chowder',
        component_group='main_dish')
    com_15 = Component.objects.create(
        name='Ginger pork',
        component_group='main_dish')
    com_16 = Component.objects.create(
        name='Lemon baked haddock',
        component_group='main_dish')
    com_17 = Component.objects.create(
        name='Macaroni and cheese',
        component_group='main_dish')
    com_18 = Component.objects.create(
        name='Meat pie',
        component_group='main_dish')
    com_19 = Component.objects.create(
        name='Moussaka',
        component_group='main_dish')
    com_20 = Component.objects.create(
        name='Mushroom rice casserole',
        component_group='main_dish')
    com_21 = Component.objects.create(
        name='Niçoise salad',
        component_group='main_dish')
    com_22 = Component.objects.create(
        name='Peanut chicken with rice noodles',
        component_group='main_dish')
    com_23 = Component.objects.create(
        name='Penne with pesto and feta',
        component_group='main_dish')
    com_24 = Component.objects.create(
        name='Roast beef',
        component_group='main_dish')
    com_25 = Component.objects.create(
        name='Salmon and egg salad',
        component_group='main_dish')
    com_26 = Component.objects.create(
        name='Sausage and cabbage',
        component_group='main_dish')
    com_27 = Component.objects.create(
        name='Sausage cassoulet',
        component_group='main_dish')
    com_28 = Component.objects.create(
        name='Seafood pilaf',
        component_group='main_dish')
    com_29 = Component.objects.create(
        name="Shepherd's pie",
        component_group='main_dish')
    com_30 = Component.objects.create(
        name='Shrimp and mushroom linguini',
        component_group='main_dish')
    com_31 = Component.objects.create(
        name='Shrimp stir-fry',
        component_group='main_dish')
    com_32 = Component.objects.create(
        name='Spaghetti with meat sauce',
        component_group='main_dish')
    com_33 = Component.objects.create(
        name='Turkey with mushroom sauce',
        component_group='main_dish')
    com_34 = Component.objects.create(
        name='Vegatable and chickpea curry',
        component_group='main_dish')
    com_35 = Component.objects.create(
        name='Vegetable fried rice',
        component_group='main_dish')
    com_36 = Component.objects.create(
        name='Vegetable lasagna',
        component_group='main_dish')
    com_37 = Component.objects.create(
        name='Vegetarian chili',
        component_group='main_dish')
    com_38 = Component.objects.create(
        name='Vegetarian quiche',
        component_group='main_dish')
    com_39 = Component.objects.create(
        name='Walnut and cranberry rice salad',
        component_group='main_dish')
    #
    print_rows(Component.objects)

    # -----------------------------------------------------------------------------
    com_ing_1 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_101)
    com_ing_2 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_69)
    com_ing_3 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_129)
    com_ing_4 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_138)
    com_ing_5 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_150)
    com_ing_6 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_152)
    com_ing_7 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_49)
    com_ing_8 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_90)
    com_ing_9 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_51)
    com_ing_10 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_166)
    com_ing_11 = Component_ingredient.objects.create(
        component=com_1,
        ingredient=ing_61)
    com_ing_12 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_54)
    com_ing_13 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_9)
    com_ing_14 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_11)
    com_ing_15 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_138)
    com_ing_16 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_72)
    com_ing_17 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_13)
    com_ing_18 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_152)
    com_ing_19 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_48)
    com_ing_20 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_166)
    com_ing_21 = Component_ingredient.objects.create(
        component=com_2,
        ingredient=ing_169)
    com_ing_22 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_54)
    com_ing_23 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_55)
    com_ing_24 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_11)
    com_ing_25 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_138)
    com_ing_26 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_72)
    com_ing_27 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_13)
    com_ing_28 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_152)
    com_ing_29 = Component_ingredient.objects.create(
        component=com_3,
        ingredient=ing_166)
    com_ing_30 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_119)
    com_ing_31 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_124)
    com_ing_32 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_2)
    com_ing_33 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_85)
    com_ing_34 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_138)
    com_ing_35 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_147)
    com_ing_36 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_87)
    com_ing_37 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_88)
    com_ing_38 = Component_ingredient.objects.create(
        component=com_4,
        ingredient=ing_152)
    com_ing_39 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_70)
    com_ing_40 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_129)
    com_ing_41 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_138)
    com_ing_42 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_139)
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_152)
    com_ing_41 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_107)
    com_ing_42 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_108)
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_118)
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_112)
    com_ing_44 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_110)
    com_ing_45 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_163)
    com_ing_46 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_166)
    com_ing_47 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_15)
    com_ing_48 = Component_ingredient.objects.create(
        component=com_5,
        ingredient=ing_169)
    com_ing_49 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_55)
    com_ing_50 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_131)
    com_ing_51 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_71)
    com_ing_52 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_84)
    com_ing_53 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_138)
    com_ing_54 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_152)
    com_ing_55 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_50)
    com_ing_56 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_79)
    com_ing_57 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_31)
    com_ing_58 = Component_ingredient.objects.create(
        component=com_6,
        ingredient=ing_166)
    com_ing_59 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_129)
    com_ing_60 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_131)
    com_ing_61 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_71)
    com_ing_62 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_11)
    com_ing_63 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_138)
    com_ing_64 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_13)
    com_ing_65 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_152)
    com_ing_66 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_49)
    com_ing_67 = Component_ingredient.objects.create(
        component=com_7,
        ingredient=ing_61)
    com_ing_68 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_100)
    com_ing_69 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_103)
    com_ing_70 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_71)
    com_ing_71 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_105)
    com_ing_72 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_106)
    com_ing_73 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_107)
    com_ing_74 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_57)
    com_ing_75 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_137)
    com_ing_76 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_138)
    com_ing_77 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_139)
    com_ing_78 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_143)
    com_ing_79 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_146)
    com_ing_80 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_152)
    com_ing_81 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_114)
    com_ing_82 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_60)
    com_ing_83 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_163)
    com_ing_84 = Component_ingredient.objects.create(
        component=com_8,
        ingredient=ing_166)
    com_ing_85 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_54)
    com_ing_86 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_40)
    com_ing_87 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_46)
    com_ing_88 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_11)
    com_ing_89 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_138)
    com_ing_90 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_146)
    com_ing_91 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_49)
    com_ing_92 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_155)
    com_ing_93 = Component_ingredient.objects.create(
        component=com_9,
        ingredient=ing_15)
    com_ing_94 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_129)
    com_ing_95 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_71)
    com_ing_96 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_84)
    com_ing_97 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_138)
    com_ing_98 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_150)
    com_ing_99 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_152)
    com_ing_100 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_49)
    com_ing_101 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_51)
    com_ing_102 = Component_ingredient.objects.create(
        component=com_10,
        ingredient=ing_61)
    com_ing_103 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_1)
    com_ing_104 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_38)
    com_ing_105 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_129)
    com_ing_106 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_46)
    com_ing_107 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_138)
    com_ing_108 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_146)
    com_ing_109 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_13)
    com_ing_110 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_152)
    com_ing_111 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_166)
    com_ing_112 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_61)
    com_ing_113 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_37)
    com_ing_114 = Component_ingredient.objects.create(
        component=com_11,
        ingredient=ing_169)
    com_ing_115 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_38)
    com_ing_116 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_129)
    com_ing_117 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_46)
    com_ing_118 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_11)
    com_ing_119 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_146)
    com_ing_120 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_147)
    com_ing_121 = Component_ingredient.objects.create(
        component=com_12,
        ingredient=ing_116)
    com_ing_122 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_129)
    com_ing_123 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_131)
    com_ing_124 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_4)
    com_ing_125 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_9)
    com_ing_126 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_137)
    com_ing_127 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_138)
    com_ing_128 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_152)
    com_ing_129 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_90)
    com_ing_130 = Component_ingredient.objects.create(
        component=com_13,
        ingredient=ing_166)
    com_ing_131 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_102)
    com_ing_132 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_1)
    com_ing_133 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_134)
    com_ing_134 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_138)
    com_ing_135 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_41)
    com_ing_136 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_13)
    com_ing_137 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_152)
    com_ing_138 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_155)
    com_ing_139 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_51)
    com_ing_140 = Component_ingredient.objects.create(
        component=com_14,
        ingredient=ing_61)
    com_ing_141 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_129)
    com_ing_142 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_20)
    com_ing_143 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_138)
    com_ing_144 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_139)
    com_ing_145 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_73)
    com_ing_146 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_152)
    com_ing_147 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_28)
    com_ing_148 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_93)
    com_ing_149 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_94)
    com_ing_150 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_29)
    com_ing_151 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_95)
    com_ing_152 = Component_ingredient.objects.create(
        component=com_15,
        ingredient=ing_169)
    com_ing_153 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_102)
    com_ing_154 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_54)
    com_ing_155 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_4)
    com_ing_156 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_11)
    com_ing_157 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_138)
    com_ing_158 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_41)
    com_ing_159 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_146)
    com_ing_160 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_13)
    com_ing_161 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_152)
    com_ing_162 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_155)
    com_ing_163 = Component_ingredient.objects.create(
        component=com_16,
        ingredient=ing_116)
    com_ing_164 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_54)
    com_ing_165 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_1)
    com_ing_166 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_130)
    com_ing_167 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_4)
    com_ing_168 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_9)
    com_ing_169 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_44)
    com_ing_170 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_138)
    com_ing_171 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_141)
    com_ing_172 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_13)
    com_ing_173 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_150)
    com_ing_174 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_112)
    com_ing_175 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_113)
    com_ing_176 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_152)
    com_ing_177 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_49)
    com_ing_178 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_32)
    com_ing_179 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_61)
    com_ing_180 = Component_ingredient.objects.create(
        component=com_17,
        ingredient=ing_35)
    com_ing_181 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_102)
    com_ing_182 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_54)
    com_ing_183 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_105)
    com_ing_184 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_138)
    com_ing_185 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_72)
    com_ing_186 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_73)
    com_ing_187 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_113)
    com_ing_188 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_152)
    com_ing_189 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_155)
    com_ing_190 = Component_ingredient.objects.create(
        component=com_18,
        ingredient=ing_61)
    com_ing_191 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_100)
    com_ing_192 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_1)
    com_ing_193 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_9)
    com_ing_194 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_105)
    com_ing_195 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_137)
    com_ing_196 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_11)
    com_ing_197 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_138)
    com_ing_198 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_72)
    com_ing_199 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_13)
    com_ing_200 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_152)
    com_ing_201 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_48)
    com_ing_202 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_166)
    com_ing_203 = Component_ingredient.objects.create(
        component=com_19,
        ingredient=ing_61)
    com_ing_204 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_102)
    com_ing_205 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_55)
    com_ing_206 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_129)
    com_ing_207 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_9)
    com_ing_208 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_138)
    com_ing_209 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_145)
    com_ing_210 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_47)
    com_ing_211 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_150)
    com_ing_212 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_49)
    com_ing_213 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_116)
    com_ing_214 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_32)
    com_ing_215 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_97)
    com_ing_216 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_63)
    com_ing_217 = Component_ingredient.objects.create(
        component=com_20,
        ingredient=ing_169)
    com_ing_218 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_39)
    com_ing_219 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_129)
    com_ing_220 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_85)
    com_ing_221 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_138)
    com_ing_222 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_140)
    com_ing_223 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_146)
    com_ing_224 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_151)
    com_ing_225 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_152)
    com_ing_226 = Component_ingredient.objects.create(
        component=com_21,
        ingredient=ing_116)
    com_ing_227 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_129)
    com_ing_228 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_71)
    com_ing_229 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_45)
    com_ing_230 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_107)
    com_ing_231 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_138)
    com_ing_232 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_139)
    com_ing_233 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_86)
    com_ing_234 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_148)
    com_ing_235 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_152)
    com_ing_236 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_24)
    com_ing_237 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_25)
    com_ing_238 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_28)
    com_ing_239 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_94)
    com_ing_240 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_95)
    com_ing_241 = Component_ingredient.objects.create(
        component=com_22,
        ingredient=ing_118)
    com_ing_242 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_43)
    com_ing_243 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_129)
    com_ing_244 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_6)
    com_ing_245 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_9)
    com_ing_246 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_138)
    com_ing_247 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_146)
    com_ing_248 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_147)
    com_ing_249 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_152)
    com_ing_250 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_32)
    com_ing_251 = Component_ingredient.objects.create(
        component=com_23,
        ingredient=ing_36)
    com_ing_252 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_129)
    com_ing_253 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_131)
    com_ing_254 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_85)
    com_ing_255 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_138)
    com_ing_256 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_150)
    com_ing_257 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_152)
    com_ing_258 = Component_ingredient.objects.create(
        component=com_24,
        ingredient=ing_78)
    com_ing_259 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_38)
    com_ing_260 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_129)
    com_ing_261 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_46)
    com_ing_262 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_11)
    com_ing_263 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_138)
    com_ing_264 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_146)
    com_ing_265 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_147)
    com_ing_266 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_88)
    com_ing_267 = Component_ingredient.objects.create(
        component=com_25,
        ingredient=ing_152)
    com_ing_268 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_102)
    com_ing_269 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_128)
    com_ing_270 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_138)
    com_ing_271 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_152)
    com_ing_272 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_154)
    com_ing_273 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_77)
    com_ing_274 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_155)
    com_ing_275 = Component_ingredient.objects.create(
        component=com_26,
        ingredient=ing_97)
    com_ing_276 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_101)
    com_ing_277 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_129)
    com_ing_278 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_131)
    com_ing_279 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_138)
    com_ing_280 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_152)
    com_ing_281 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_49)
    com_ing_282 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_77)
    com_ing_283 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_51)
    com_ing_284 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_166)
    com_ing_285 = Component_ingredient.objects.create(
        component=com_27,
        ingredient=ing_68)
    com_ing_286 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_55)
    com_ing_287 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_131)
    com_ing_288 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_138)
    com_ing_289 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_152)
    com_ing_290 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_114)
    com_ing_291 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_49)
    com_ing_292 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_98)
    com_ing_293 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_166)
    com_ing_294 = Component_ingredient.objects.create(
        component=com_28,
        ingredient=ing_97)
    com_ing_295 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_54)
    com_ing_296 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_105)
    com_ing_297 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_106)
    com_ing_298 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_134)
    com_ing_299 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_138)
    com_ing_300 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_72)
    com_ing_301 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_13)
    com_ing_302 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_113)
    com_ing_303 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_152)
    com_ing_304 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_114)
    com_ing_305 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_155)
    com_ing_306 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_115)
    com_ing_307 = Component_ingredient.objects.create(
        component=com_29,
        ingredient=ing_51)
    com_ing_308 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_1)
    com_ing_309 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_138)
    com_ing_310 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_13)
    com_ing_311 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_152)
    com_ing_312 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_99)
    com_ing_313 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_116)
    com_ing_314 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_166)
    com_ing_315 = Component_ingredient.objects.create(
        component=com_30,
        ingredient=ing_61)
    com_ing_316 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_126)
    com_ing_317 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_129)
    com_ing_318 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_138)
    com_ing_319 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_139)
    com_ing_320 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_142)
    com_ing_321 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_149)
    com_ing_322 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_150)
    com_ing_323 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_152)
    com_ing_324 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_159)
    com_ing_325 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_28)
    com_ing_326 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_94)
    com_ing_327 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_29)
    com_ing_328 = Component_ingredient.objects.create(
        component=com_31,
        ingredient=ing_99)
    com_ing_329 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_129)
    com_ing_330 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_9)
    com_ing_331 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_138)
    com_ing_332 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_72)
    com_ing_333 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_150)
    com_ing_334 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_152)
    com_ing_335 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_48)
    com_ing_336 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_31)
    com_ing_337 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_166)
    com_ing_338 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_37)
    com_ing_339 = Component_ingredient.objects.create(
        component=com_32,
        ingredient=ing_169)
    com_ing_340 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_55)
    com_ing_341 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_1)
    com_ing_342 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_131)
    com_ing_343 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_138)
    com_ing_344 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_150)
    com_ing_345 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_152)
    com_ing_346 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_166)
    com_ing_347 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_81)
    com_ing_348 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_61)
    com_ing_349 = Component_ingredient.objects.create(
        component=com_33,
        ingredient=ing_97)
    com_ing_350 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_43)
    com_ing_351 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_128)
    com_ing_352 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_65)
    com_ing_353 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_44)
    com_ing_354 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_45)
    com_ing_355 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_46)
    com_ing_356 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_137)
    com_ing_357 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_138)
    com_ing_358 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_141)
    com_ing_359 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_142)
    com_ing_360 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_47)
    com_ing_361 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_152)
    com_ing_362 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_48)
    com_ing_363 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_49)
    com_ing_364 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_159)
    com_ing_365 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_50)
    com_ing_366 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_163)
    com_ing_367 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_51)
    com_ing_368 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_166)
    com_ing_369 = Component_ingredient.objects.create(
        component=com_34,
        ingredient=ing_169)
    com_ing_370 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_55)
    com_ing_371 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_129)
    com_ing_372 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_45)
    com_ing_373 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_137)
    com_ing_374 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_11)
    com_ing_375 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_138)
    com_ing_376 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_139)
    com_ing_377 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_152)
    com_ing_378 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_92)
    com_ing_379 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_95)
    com_ing_380 = Component_ingredient.objects.create(
        component=com_35,
        ingredient=ing_169)
    com_ing_381 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_4)
    com_ing_382 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_9)
    com_ing_383 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_137)
    com_ing_384 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_138)
    com_ing_385 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_150)
    com_ing_386 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_152)
    com_ing_387 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_48)
    com_ing_388 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_162)
    com_ing_389 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_166)
    com_ing_390 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_33)
    com_ing_391 = Component_ingredient.objects.create(
        component=com_36,
        ingredient=ing_169)
    com_ing_392 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_129)
    com_ing_393 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_65)
    com_ing_394 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_104)
    com_ing_395 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_134)
    com_ing_396 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_108)
    com_ing_397 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_138)
    com_ing_398 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_66)
    com_ing_399 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_152)
    com_ing_400 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_48)
    com_ing_401 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_164)
    com_ing_402 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_166)
    com_ing_403 = Component_ingredient.objects.create(
        component=com_37,
        ingredient=ing_169)
    com_ing_404 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_43)
    com_ing_405 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_4)
    com_ing_406 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_44)
    com_ing_407 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_45)
    com_ing_408 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_46)
    com_ing_409 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_11)
    com_ing_410 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_138)
    com_ing_411 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_47)
    com_ing_412 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_13)
    com_ing_413 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_152)
    com_ing_414 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_48)
    com_ing_415 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_49)
    com_ing_416 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_50)
    com_ing_417 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_162)
    com_ing_418 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_163)
    com_ing_419 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_51)
    com_ing_420 = Component_ingredient.objects.create(
        component=com_38,
        ingredient=ing_61)
    com_ing_421 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_82)
    com_ing_422 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_43)
    com_ing_423 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_55)
    com_ing_424 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_7)
    com_ing_425 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_135)
    com_ing_426 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_138)
    com_ing_427 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_147)
    com_ing_428 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_88)
    com_ing_429 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_152)
    com_ing_430 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_49)
    com_ing_431 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_32)
    com_ing_432 = Component_ingredient.objects.create(
        component=com_39,
        ingredient=ing_63)
    #
    print_rows(Component_ingredient.objects.filter(date=None))

    # -----------------------------------------------------------------------------
    ri_1 = Restricted_item.objects.create(
        name='fish',
        restricted_item_group='fish&seafood',
        description='')
    ri_2 = Restricted_item.objects.create(
        name='seafood',
        restricted_item_group='fish&seafood',
        description='')
    ri_3 = Restricted_item.objects.create(
        name='beef',
        restricted_item_group='meat',
        description='')
    ri_4 = Restricted_item.objects.create(
        name='lamb',
        restricted_item_group='meat',
        description='gluten, salt')
    ri_5 = Restricted_item.objects.create(
        name='pork',
        restricted_item_group='meat',
        description='text')
    ri_6 = Restricted_item.objects.create(
        name='poultry',
        restricted_item_group='meat',
        description='text')
    ri_7 = Restricted_item.objects.create(
        name='sausage',
        restricted_item_group='meat',
        description='text')
    ri_8 = Restricted_item.objects.create(
        name='turkey ',
        restricted_item_group='meat',
        description='text')
    ri_9 = Restricted_item.objects.create(
        name='nuts ',
        restricted_item_group='nuts&seeds',
        description='')
    ri_10 = Restricted_item.objects.create(
        name='peanut butter',
        restricted_item_group='nuts&seeds',
        description='')
    ri_11 = Restricted_item.objects.create(
        name='peanuts',
        restricted_item_group='nuts&seeds',
        description='gluten, salt')
    ri_12 = Restricted_item.objects.create(
        name='seeds',
        restricted_item_group='nuts&seeds',
        description='text')
    ri_13 = Restricted_item.objects.create(
        name='dairy',
        restricted_item_group='other',
        description='text')
    ri_14 = Restricted_item.objects.create(
        name='gluten',
        restricted_item_group='other',
        description='text')
    ri_15 = Restricted_item.objects.create(
        name='spicy',
        restricted_item_group='other',
        description='text')
    ri_16 = Restricted_item.objects.create(
        name='green veggies',
        restricted_item_group='vegetables',
        description='')
    ri_17 = Restricted_item.objects.create(
        name='leafy greens',
        restricted_item_group='vegetables',
        description='')
    ri_18 = Restricted_item.objects.create(
        name='legumes',
        restricted_item_group='vegetables',
        description='gluten, salt')
    #
    print_rows(Restricted_item.objects)

    # -----------------------------------------------------------------------------
    inc_1 = Incompatibility.objects.create(
        restricted_item=ri_3, ingredient=ing_69)
    inc_2 = Incompatibility.objects.create(
        restricted_item=ri_3, ingredient=ing_72)
    inc_3 = Incompatibility.objects.create(
        restricted_item=ri_3, ingredient=ing_78)
    inc_4 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_1)
    inc_5 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_2)
    inc_6 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_3)
    inc_7 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_4)
    inc_8 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_5)
    inc_9 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_6)
    inc_10 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_7)
    inc_11 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_8)
    inc_12 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_9)
    inc_13 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_10)
    inc_14 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_13)
    inc_15 = Incompatibility.objects.create(
        restricted_item=ri_13, ingredient=ing_15)
    inc_16 = Incompatibility.objects.create(
        restricted_item=ri_1, ingredient=ing_38)
    inc_17 = Incompatibility.objects.create(
        restricted_item=ri_1, ingredient=ing_39)
    inc_18 = Incompatibility.objects.create(
        restricted_item=ri_1, ingredient=ing_40)
    inc_19 = Incompatibility.objects.create(
        restricted_item=ri_1, ingredient=ing_41)
    inc_20 = Incompatibility.objects.create(
        restricted_item=ri_1, ingredient=ing_42)
    inc_21 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_52)
    inc_22 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_54)
    inc_23 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_57)
    inc_24 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_95)
    inc_25 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_61)
    inc_26 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_62)
    inc_27 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_33)
    inc_28 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_34)
    inc_29 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_35)
    inc_30 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_36)
    inc_31 = Incompatibility.objects.create(
        restricted_item=ri_14, ingredient=ing_37)
    inc_32 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_121)
    inc_33 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_125)
    inc_34 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_126)
    inc_35 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_127)
    inc_36 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_133)
    inc_37 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_136)
    inc_38 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_140)
    inc_39 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_141)
    inc_40 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_142)
    inc_41 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_143)
    inc_42 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_147)
    inc_43 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_160)
    inc_44 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_161)
    inc_45 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_162)
    inc_46 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_165)
    inc_47 = Incompatibility.objects.create(
        restricted_item=ri_16, ingredient=ing_169)
    inc_48 = Incompatibility.objects.create(
        restricted_item=ri_4, ingredient=ing_75)
    inc_49 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_121)
    inc_50 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_133)
    inc_51 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_143)
    inc_52 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_147)
    inc_53 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_162)
    inc_54 = Incompatibility.objects.create(
        restricted_item=ri_17, ingredient=ing_165)
    inc_55 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_64)
    inc_56 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_65)
    inc_57 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_66)
    inc_58 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_67)
    inc_59 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_14)
    inc_60 = Incompatibility.objects.create(
        restricted_item=ri_18, ingredient=ing_68)
    inc_61 = Incompatibility.objects.create(
        restricted_item=ri_9, ingredient=ing_16)
    inc_62 = Incompatibility.objects.create(
        restricted_item=ri_9, ingredient=ing_17)
    inc_63 = Incompatibility.objects.create(
        restricted_item=ri_9, ingredient=ing_24)
    inc_64 = Incompatibility.objects.create(
        restricted_item=ri_9, ingredient=ing_25)
    inc_65 = Incompatibility.objects.create(
        restricted_item=ri_9, ingredient=ing_32)
    inc_66 = Incompatibility.objects.create(
        restricted_item=ri_10, ingredient=ing_24)
    inc_67 = Incompatibility.objects.create(
        restricted_item=ri_11, ingredient=ing_24)
    inc_68 = Incompatibility.objects.create(
        restricted_item=ri_11, ingredient=ing_25)
    inc_69 = Incompatibility.objects.create(
        restricted_item=ri_5, ingredient=ing_73)
    inc_70 = Incompatibility.objects.create(
        restricted_item=ri_5, ingredient=ing_74)
    inc_71 = Incompatibility.objects.create(
        restricted_item=ri_5, ingredient=ing_77)
    inc_72 = Incompatibility.objects.create(
        restricted_item=ri_6, ingredient=ing_71)
    inc_73 = Incompatibility.objects.create(
        restricted_item=ri_6, ingredient=ing_81)
    inc_74 = Incompatibility.objects.create(
        restricted_item=ri_7, ingredient=ing_77)
    inc_75 = Incompatibility.objects.create(
        restricted_item=ri_2, ingredient=ing_98)
    inc_76 = Incompatibility.objects.create(
        restricted_item=ri_2, ingredient=ing_99)
    inc_77 = Incompatibility.objects.create(
        restricted_item=ri_12, ingredient=ing_30)
    inc_78 = Incompatibility.objects.create(
        restricted_item=ri_15, ingredient=ing_104)
    inc_79 = Incompatibility.objects.create(
        restricted_item=ri_15, ingredient=ing_109)
    inc_80 = Incompatibility.objects.create(
        restricted_item=ri_8, ingredient=ing_81)
    #
    print_rows(Incompatibility.objects)

    # =====================================================
    print("END dataload")



import datetime

from django.contrib.auth.models import User
from django.test import TestCase

from souschef.meal.models import Component, Component_ingredient
from souschef.meal.models import Ingredient, Incompatibility
from souschef.meal.models import Menu, Menu_component, Restricted_item


class ComponentTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
         Component.objects.create(
            name='Coq au vin', component_group='main dish')

    def test_str_is_fullname(self):
        """Component's string representation is its name"""
        name = 'Coq au vin'
        component = Component.objects.get(name=name)
        self.assertEqual(name, str(component))


class Component_ingredientTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        component = Component.objects.create(
            name='Coq au vin', component_group='main dish')
        ingredient = Ingredient.objects.create(
            name='Green peas', ingredient_group='veggies and fruits')
        Component_ingredient.objects.create(component=component,
                                            ingredient=ingredient)

    def test_str_includes_all_names(self):
        """A Component_ingredient's string representation includes the name
        of the component and the name of the ingredient.
        """
        component = Component.objects.get(name='Coq au vin')
        ingredient = Ingredient.objects.get(name='Green peas')
        component_ingredient = Component_ingredient.objects.get(
            component=component, ingredient=ingredient)
        self.assertTrue(component.name in str(component_ingredient))
        self.assertTrue(ingredient.name in str(component_ingredient))


class IncompatibilityTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        restricted_item = Restricted_item.objects.create(
            name='pork', restricted_item_group='meat')
        ingredient = Ingredient.objects.create(
            name='ham', ingredient_group='meat')
        Incompatibility.objects.create(restricted_item=restricted_item,
                                       ingredient=ingredient)

    def test_str_includes_all_names(self):
        """Incompatibility's string representation includes the name
        of the restricted_item and the name of the ingredient.
        """
        restricted_item = Restricted_item.objects.get(name='pork')
        ingredient = Ingredient.objects.get(name='ham')
        incompatibility = Incompatibility.objects.get(
            restricted_item=restricted_item, ingredient=ingredient)
        self.assertTrue(restricted_item.name in str(incompatibility))
        self.assertTrue(ingredient.name in str(incompatibility))


class IngredientTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
         Ingredient.objects.create(
            name='Green peas', ingredient_group='veggies and fruits')

    def test_str_is_fullname(self):
        """Ingredient's string representation is its name"""
        name = 'Green peas'
        ingredient = Ingredient.objects.get(name=name)
        self.assertEqual(name, str(ingredient))


class MenuTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
         Menu.objects.create(date=datetime.date(2016, 5, 21))

    def test_str_is_fullname(self):
        """Menu's string representation includes date"""
        date = datetime.date(2016, 5, 21)
        menu = Menu.objects.get(date=date)
        self.assertTrue(str(date) in str(menu))


class Menu_componentTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        menu = Menu.objects.create(date=datetime.date(2016, 5, 21))
        component = Component.objects.create(
            name='Coq au vin', component_group='main dish')
        Menu_component.objects.create(menu=menu, component=component)

    def test_str_includes_all_names(self):
        """A Menu_component's string representation includes the date
        of the menu and the name of the component.
        """
        date = datetime.date(2016, 5, 21)
        menu = Menu.objects.get(date=date)
        component = Component.objects.get(name='Coq au vin')
        menu_component = Menu_component.objects.get(
            menu=menu, component=component)
        self.assertTrue(str(date) in str(menu_component))
        self.assertTrue(component.name in str(menu_component))


class CreateMenuAndComponentsTestCase(TestCase):

    fixtures = ['meal_initial_data']

    def test_menu_new(self):
        number = Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            ['Ginger pork',
             'Green Salad', 'Fruit Salad', 'Day s Dessert',
             'Day s Diabetic Dessert', 'Day s Pudding', 'Day s Compote'])
        self.assertEqual(number, 7)

    def test_menu_existed(self):
        Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            ['Ginger pork',
             'Green Salad', 'Fruit Salad', 'Day s Dessert',
             'Day s Diabetic Dessert', 'Day s Pudding', 'Day s Compote'])
        number = Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            ['Coq au vin',
             'Green Salad', 'Fruit Salad', 'Day s Dessert',
             'Day s Diabetic Dessert', 'Day s Pudding'])
        self.assertEqual(number, 6)

    def test_dish_does_not_exist_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                ['Chateaubriand Steak',
                 'Green Salad', 'Fruit Salad', 'Day s Dessert',
                 'Day s Diabetic Dessert', 'Day s Pudding', 'Day s Compote'])
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)

    def test_many_dishes_per_component_group_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                ['Ginger pork', 'Coq au vin',
                 'Green Salad', 'Fruit Salad', 'Day s Dessert',
                 'Day s Diabetic Dessert', 'Day s Pudding', 'Day s Compote'])
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)

    def test_no_main_dish_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                ['Green Salad', 'Fruit Salad', 'Day s Dessert',
                 'Day s Diabetic Dessert', 'Day s Pudding', 'Day s Compote'])
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)


class Restricted_itemTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
         Restricted_item.objects.create(
            name='pork', restricted_item_group='meat')

    def test_str_is_fullname(self):
        """Restricted_Item's string representation is its name"""
        name = 'pork'
        restricted_item = Restricted_item.objects.get(name=name)
        self.assertEqual(name, str(restricted_item))


class MealAdminTestCase(TestCase):

    fixtures = ['sample_data']

    @classmethod
    def setUpTestData(cls):
        cls.admin = User.objects.create_superuser(
            username='admin@example.com',
            email='admin@example.com',
            password='test1234'
        )

    def setUp(self):
        self.client.login(username=self.admin.username, password='test1234')

    def test_component_search(self):
        response = self.client.get('/admin/meal/component/?q=ess')
        self.assertTrue(b'Diabetic' in response.content)

    def test_ingredient_search(self):
        response = self.client.get('/admin/meal/ingredient/?q=ann')
        self.assertTrue(b'Canned' in response.content)

    def test_restricted_item_search(self):
        response = self.client.get('/admin/meal/restricted_item/?q=ea')
        self.assertTrue(b'eef' in response.content)



import csv
import json

from django.http import HttpResponse
from django.http.response import HttpResponseRedirect
from django.views import generic, View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.contrib import messages
from django.urls import reverse_lazy, reverse
from django.utils import timezone
from django.shortcuts import get_object_or_404
from extra_views import CreateWithInlinesView, UpdateWithInlinesView

from datetime import datetime

from souschef.order.models import ORDER_STATUS_CANCELLED, Order, OrderFilter, \
    ORDER_STATUS, OrderStatusChange
from souschef.order.mixins import AjaxableResponseMixin, FormValidAjaxableResponseMixin
from souschef.order.forms import CreateOrderItem, UpdateOrderItem, \
    CreateOrdersBatchForm, OrderStatusChangeForm

from souschef.meal.models import COMPONENT_GROUP_CHOICES, COMPONENT_GROUP_CHOICES_SIDES
from souschef.meal.settings import COMPONENT_SYSTEM_DEFAULT
from souschef.member.models import Client, DAYS_OF_WEEK


class OrderList(
        LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    context_object_name = 'orders'
    model = Order
    paginate_by = 20
    permission_required = 'sous_chef.read'
    template_name = 'list.html'

    def get_queryset(self):
        uf = OrderFilter(self.request.GET)
        return uf.qs.select_related(
            'client__member'
        ).prefetch_related('orders')

    def get_context_data(self, **kwargs):
        uf = OrderFilter(self.request.GET, queryset=self.get_queryset())

        context = super(OrderList, self).get_context_data(**kwargs)

        context['filter'] = uf

        text = ''
        count = 0
        for getVariable in self.request.GET:
            if getVariable == "page":
                continue
            for getValue in self.request.GET.getlist(getVariable):
                if count == 0:
                    text += "?" + getVariable + "=" + getValue
                else:
                    text += "&" + getVariable + "=" + getValue
                count += 1

        text = text + "?" if count == 0 else text + "&"
        context['get'] = text

        return context

    def get(self, request, **kwargs):

        self.format = request.GET.get('format', False)

        if self.format == 'csv':
            return ExportCSV(
                self, self.get_queryset()
            )

        return super(OrderList, self).get(request, **kwargs)


class OrderDetail(
        LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView):
    model = Order
    permission_required = 'sous_chef.read'
    template_name = 'view.html'
    context_object_name = 'order'
    queryset = Order.objects.all().prefetch_related(
        'orders'
    )

    def get_context_data(self, **kwargs):
        context = super(OrderDetail, self).get_context_data(**kwargs)
        context['status'] = ORDER_STATUS
        return context


class CreateOrder(
        AjaxableResponseMixin, LoginRequiredMixin,
        PermissionRequiredMixin, CreateWithInlinesView):
    fields = ['client', 'delivery_date']
    inlines = [CreateOrderItem]
    model = Order
    permission_required = 'sous_chef.edit'
    template_name = 'create.html'

    # TODO: Change the validation of the form,
    # If component is present, then component_group is not required
    # and vice versa

    def get_success_url(self):
        return self.object.get_absolute_url()


class CreateOrdersBatch(
        LoginRequiredMixin, PermissionRequiredMixin, generic.FormView):
    permission_required = 'sous_chef.edit'
    success_url = reverse_lazy('order:create_batch')
    template_name = "order/create_batch.html"

    def get_form(self, *args, **kwargs):
        k = self.get_form_kwargs()  # kwargs to initialize the form
        if self.request.method == "POST" and \
           self.request.POST.get('delivery_dates'):
            k['delivery_dates'] = self.request.POST[
                'delivery_dates'
            ].split('|')

        return CreateOrdersBatchForm(**k)

    def get_context_data(self, **kwargs):
        context = super(CreateOrdersBatch, self).get_context_data(**kwargs)
        # Define here any needed variable for template
        context["meals"] = list(filter(
            lambda tup: tup[0] != COMPONENT_GROUP_CHOICES_SIDES,
            COMPONENT_GROUP_CHOICES
        ))

        # delivery_dates
        if self.request.method == "POST" and \
           self.request.POST.get('delivery_dates'):
            delivery_dates = self.request.POST['delivery_dates'].split('|')
        else:
            delivery_dates = []

        # dates of orders to override
        if self.request.method == "POST" and \
           self.request.POST.get('override_dates'):
            override_dates = self.request.POST['override_dates'].split('|')
            override_dates = [x for x in override_dates if x in delivery_dates]
        else:
            override_dates = []

        # inactive accordion dates
        if self.request.method == "POST" and \
           self.request.POST.get('accordions_inactive'):
            context[
                'accordions_inactive'
            ] = self.request.POST['accordions_inactive'].split('|')
        else:
            context['accordions_inactive'] = []

        if self.request.method == "POST" and \
           self.request.POST.get('client'):
            c = Client.objects.get(pk=self.request.POST['client'])
            meals_default_dict = dict(c.meals_default)
            context['client'] = c

            # The dates where an order already exists.
            today = timezone.datetime.today()
            ordered_dates = c.orders.filter(
                delivery_date__gte=today
            ).exclude(status='C').order_by('delivery_date').values_list(
                'delivery_date', flat=True
            )
            context['ordered_dates'] = '|'.join(map(
                lambda d: d.strftime('%Y-%m-%d'),
                ordered_dates
            ))
        else:
            meals_default_dict = dict(
                map(
                    lambda x: (x[0], None),
                    DAYS_OF_WEEK
                ),
            )

        context['delivery_dates'] = []
        context['override_orders'] = []
        for date in delivery_dates:
            date_obj = datetime.strptime(date, '%Y-%m-%d')
            # sunday = 0, saturday = 6
            day = ("sunday", "monday", "tuesday", "wednesday", "thursday",
                   "friday", "saturday")[int(date_obj.strftime('%w'))]

            if date not in override_dates and context.get('client'):
                order_on_day = context['client'].orders\
                    .filter(delivery_date=date_obj).exclude(status='C').first()
                if context.get('client') and order_on_day:
                    # the client has an active order already on this day.
                    # show the warning modal if it's not already being shown.
                    context['show_override_modal'] = True
                    context['override_orders'].append(order_on_day)

            if not meals_default_dict[day]:  # None or {}
                # system default
                default_json = json.dumps(
                    COMPONENT_SYSTEM_DEFAULT
                )
            else:
                # client default
                default_json = json.dumps(meals_default_dict[day])

            context['delivery_dates'].append(
                (date, date_obj, default_json)
            )

        context['override_dates'] = '|'.join(override_dates)
        return context

    def form_invalid(self, form, **kwargs):
        # open accordions if there's an invalid field in it.
        context = self.get_context_data(**kwargs)
        context['form'] = form
        if form.cleaned_data.get('is_submit') != 1:
            form._errors = {}
        else:
            for field in form.errors.keys():
                try:
                    # next() - find first occurence
                    invalid_date = next(
                        date for date in context['accordions_inactive']
                        if date in field
                    )
                    print(invalid_date)
                    context['accordions_inactive'].remove(invalid_date)
                except StopIteration:
                    pass

        return self.render_to_response(context)

    def form_valid(self, form):
        # Get posted datas
        del_dates = form.cleaned_data['delivery_dates'].split('|')
        ovr_dates = form.cleaned_data['override_dates'].split('|')
        client = form.cleaned_data['client']
        items = form.cleaned_data
        del items['delivery_dates']
        del items['client']

        # Place orders using posted datas
        created_orders = Order.objects.create_batch_orders(
            del_dates, client, items, override_dates=ovr_dates
        )

        # check created and uncreated dates
        created_dates = []
        for order in created_orders:
            d = order.delivery_date.strftime('%Y-%m-%d')
            created_dates.append(d)
        uncreated_dates = []
        for d in del_dates:
            if d not in created_dates:
                uncreated_dates.append(d)

        # Alert user on order placement
        if created_dates:
            messages.add_message(
                self.request, messages.SUCCESS,
                (_('%(n)s order(s) successfully placed '
                   'for %(client)s on %(dates)s.') % {
                       'n': len(created_dates),
                       'client': client,
                       'dates': ', '.join(created_dates)
                })
            )
        if uncreated_dates:
            messages.add_message(
                self.request, messages.WARNING,
                (_('%(n)s existing order(s) skipped '
                   'for %(client)s on %(dates)s.') % {
                       'n': len(uncreated_dates),
                       'client': client,
                       'dates': ', '.join(uncreated_dates)
                })
            )

        response = super(CreateOrdersBatch, self).form_valid(form)
        return response


class UpdateOrder(
        LoginRequiredMixin, PermissionRequiredMixin,
        AjaxableResponseMixin, UpdateWithInlinesView):
    fields = ['client', 'delivery_date']
    inlines = [UpdateOrderItem]
    model = Order
    permission_required = 'sous_chef.edit'
    template_name = 'update.html'

    def get_form(self, *args, **kwargs):
        form = super(UpdateOrder, self).get_form(*args, **kwargs)
        form.fields['client'].queryset = Client.objects.all().select_related(
            'member'
        ).only(
            'member__firstname',
            'member__lastname'
        )
        return form

    def get_success_url(self):
        return self.object.get_absolute_url()


class UpdateOrderStatus(
        LoginRequiredMixin, PermissionRequiredMixin,
        FormValidAjaxableResponseMixin, generic.CreateView):
    form_class = OrderStatusChangeForm
    model = OrderStatusChange
    permission_required = 'sous_chef.edit'
    template_name = "order_update_status.html"

    def get_context_data(self, **kwargs):
        context = super(UpdateOrderStatus, self).get_context_data(**kwargs)
        context['order'] = get_object_or_404(
            Order, pk=self.kwargs.get('pk')
        )
        context['order_status'] = ORDER_STATUS
        context['order_no_charge'] = 'N'
        context['order_no_charge_reasons'] = (
            _('Restrictions mistake'),
            _('Delivery mistake'),
            _('Really unhappy client')
        )
        return context

    def get_initial(self):
        order = get_object_or_404(Order, pk=self.kwargs.get('pk'))
        return {
            'order': self.kwargs.get('pk'),
            'status_from': order.status,
            'status_to': self.request.GET.get('status'),
        }

    def form_valid(self, form):
        response = super(UpdateOrderStatus, self).form_valid(form)
        messages.add_message(
            self.request, messages.SUCCESS,
            _("The order status was updated successfully.")
        )
        return response

    def get_success_url(self):
        return reverse(
            'order:view', kwargs={'pk': self.kwargs.get('pk')}
        )


@method_decorator(csrf_exempt, name='dispatch')
class CreateDeleteOrderClientBill(
        LoginRequiredMixin, PermissionRequiredMixin, View):
    permission_required = 'sous_chef.edit'

    def get_object(self, pk):
        return get_object_or_404(
            Order.objects.prefetch_related('orders'),
            pk=pk
        )

    def post(self, request, pk, *args, **kwargs):
        order = self.get_object(pk)
        order.includes_a_bill = True
        return HttpResponse('OK', status=200)

    def delete(self, request, pk, *args, **kwargs):
        order = self.get_object(pk)
        order.includes_a_bill = False
        return HttpResponse('OK', status=200)


class CancelOrder(
        LoginRequiredMixin, PermissionRequiredMixin, View):
    permission_required = 'sous_chef.edit'

    def post(self, request, pk, *args, **kwargs):
        order = get_object_or_404(Order, pk=pk)
        order.status = ORDER_STATUS_CANCELLED
        order.save()
        return HttpResponseRedirect(request.GET['next'])


class DeleteOrder(
        LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
    model = Order
    permission_required = 'sous_chef.edit'
    template_name = 'order_confirm_delete.html'

    def get_success_url(self):
        # 'next' parameter should always be included in POST'ed URL.
        return self.request.GET.get('next') or self.request.POST['next']


def ExportCSV(request, queryset):
    response = HttpResponse(content_type="text/csv")
    response['Content-Disposition'] =\
        'attachment; filename=order_export.csv'
    writer = csv.writer(response, csv.excel)

    writer.writerow([
        "ID",
        "Client Firstname",
        "Client Lastname",
        "Order Status",
        "Creation Date",
        "Delivery Date",
        "price",
    ])

    for obj in queryset:
        writer.writerow([
            obj.id,
            obj.client.member.firstname,
            obj.client.member.lastname,
            obj.get_status_display(),
            obj.creation_date,
            obj.delivery_date,
            obj.price,
        ])

    return response



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-03 22:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0005_fix241b'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='status',
            field=models.CharField(choices=[('O', 'Ordered'), ('D', 'Delivered'), ('N', 'No Charge'), ('B', 'Billed'), ('P', 'Paid')], max_length=1, verbose_name='order status'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-14 12:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0002_ingredient_ingredient_group'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ingredient',
            options={'verbose_name_plural': 'ingredients'},
        ),
        migrations.AlterField(
            model_name='component',
            name='component_group',
            field=models.CharField(choices=[('main dish', 'Main dish'), ('dessert', 'Dessert'), ('diabetic dessert', 'Diabetic dessert'), ('fruit salad', 'Fruit salad'), ('green salad', 'Green salad'), ('pudding', 'Pudding'), ('compote', 'Compote')], max_length=100, verbose_name='component group'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-11-22 04:58
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0011_auto_20161014_1901'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='order',
            options={'ordering': ['-delivery_date'], 'verbose_name_plural': 'orders'},
        ),
    ]



from django.contrib import admin
from souschef.note.models import Note, NotePriority, NoteCategory

class NoteAdmin(admin.ModelAdmin):
    list_display = ('date_created', 'date_modified', 'author', 'is_read', 'is_deleted', 'priority', 'category', 'client', 'note')
    list_filter = ('priority', 'category', 'is_read', 'is_deleted')


# Register your models here.
admin.site.register(Note, NoteAdmin)
admin.site.register((NotePriority, NoteCategory))



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 00:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0006_auto_20160826_0007'),
        ('member', '0013_auto_20160820_2322'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='clientscheduledstatus',
            options={'ordering': ['change_date']},
        ),
        migrations.AddField(
            model_name='client',
            name='components_to_avoid',
            field=models.ManyToManyField(through='member.Client_avoid_component', to='meal.Component'),
        ),
        migrations.AddField(
            model_name='client',
            name='ingredients_to_avoid',
            field=models.ManyToManyField(through='member.Client_avoid_ingredient', to='meal.Ingredient'),
        ),
        migrations.AddField(
            model_name='client',
            name='options',
            field=models.ManyToManyField(through='member.Client_option', to='member.Option'),
        ),
        migrations.AddField(
            model_name='client',
            name='restrictions',
            field=models.ManyToManyField(through='member.Restriction', to='meal.Restricted_item'),
        ),
        migrations.AlterField(
            model_name='clientscheduledstatus',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='scheduled_statuses', to='member.Client'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-03 20:52
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('member', '0010_auto_20160803_2052'),
    ]

    operations = [
        migrations.CreateModel(
            name='Note',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('note', models.TextField(verbose_name='Note')),
                ('date', models.DateField(default=django.utils.timezone.now, verbose_name='Date')),
                ('is_read', models.BooleanField(default=False, verbose_name='Is read')),
                ('priority', models.CharField(choices=[('normal', 'Normal'), ('urgent', 'Urgent')], default='normal', max_length=15)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Author')),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='client_notes', to='member.Client', verbose_name='Client')),
            ],
            options={
                'verbose_name_plural': 'Notes',
                'ordering': ('-date', 'note'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-15 15:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0002_auto_20160614_1736'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='client_order', to='member.Client', verbose_name='client'),
        ),
    ]



from functools import reduce

from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
import django_filters

from souschef.order.models import Order


class KitchenCountOrderFilter(django_filters.FilterSet):
    """ Defines the filters used to filter orders in the Kitchen Count. """

    client_name = django_filters.CharFilter(
        method='filter_client', label=_('Search by client name'))

    class Meta:
        model = Order
        fields = '__all__'

    def filter_client(self, queryset, field_name, value):
        """ Filters the orders using client names. """
        if not value:
            return queryset

        bits = value.split(' ')
        queryset = queryset.filter(reduce(
            lambda q, name: q | Q(client__member__firstname__icontains=name) |
            Q(client__member__lastname__icontains=name), bits, Q()))

        return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-14 19:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0010_auto_20161012_1921'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='status',
            field=models.CharField(choices=[('O', 'Ordered'), ('D', 'Delivered'), ('N', 'No Charge'), ('C', 'Cancelled'), ('B', 'Billed'), ('P', 'Paid')], default='O', max_length=1, verbose_name='order status'),
        ),
        migrations.AlterField(
            model_name='order_item',
            name='component_group',
            field=models.CharField(blank=True, choices=[('main_dish', 'Main Dish'), ('dessert', 'Dessert'), ('diabetic', 'Diabetic Dessert'), ('fruit_salad', 'Fruit Salad'), ('green_salad', 'Green Salad'), ('pudding', 'Pudding'), ('compote', 'Compote')], max_length=100, null=True, verbose_name='component group'),
        ),
        migrations.AlterField(
            model_name='order_item',
            name='order_item_type',
            field=models.CharField(choices=[('meal_component', 'Meal component (main dish, vegetable, side dish, seasonal)'), ('delivery', 'Delivery (general store item, invitation, ...)'), ('pickup', 'Pickup (payment)'), ('visit', 'Visit')], max_length=20, verbose_name='order item type'),
        ),
        migrations.AlterField(
            model_name='order_item',
            name='remark',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='remark'),
        ),
        migrations.AlterField(
            model_name='order_item',
            name='total_quantity',
            field=models.IntegerField(blank=True, null=True, verbose_name='total quantity'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-15 21:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0002_auto_20160605_1609'),
    ]

    operations = [
        migrations.CreateModel(
            name='Route',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
            ],
            options={
                'verbose_name_plural': 'Routes',
            },
        ),
        migrations.AlterField(
            model_name='client',
            name='birthdate',
            field=models.DateField(blank=True, default=django.utils.timezone.now, null=True),
        ),
        migrations.AlterField(
            model_name='client',
            name='gender',
            field=models.CharField(blank=True, choices=[('', 'Gender'), ('F', 'Female'), ('M', 'Male')], default='U', max_length=1, null=True),
        ),
        migrations.AddField(
            model_name='client',
            name='route',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='member.Route', verbose_name='route'),
        ),
    ]
